/*****************************************************************************
 * Copyright (c) 2006, Freescale Semiconductor                               *
 * Freescale Quick Note                                                      *
 *                                                                           *
 * File name   : main.c                                                      *
 * Project name: PWM demo software                                           *
 *                                                                           *
 * Author      : Paolo Alcantara                                             *
 * Department  : RTAC Americas                                               *
 *                                                                           *
 * Description : Show 8 PWM signals in M5213EVB pins	(4 using EVB LEDs)   *  
 *                                                                           *
 * History     :                                                             *
 * 13/11/2006  : First version of the software                               *
 *                                                                           *
 *****************************************************************************/

#include "common.h"
#include <stdio.h>

/*********************************DEFINITIONS*********************************/ 

/* PWM Prescaler Values definitions */
#define BUSCLK_1			0x0
#define BUSCLK_2			0x1
#define BUSCLK_4			0x2
#define BUSCLK_8			0x3
#define BUSCLK_16			0x4
#define BUSCLK_32			0x5
#define BUSCLK_64			0X6
#define BUSCLK_128			0x7

/* PWM Polarity and alignment */
#define LOWPOL				0x00
#define HIGHPOL				0x01
#define LEFTALIGNED			0x00
#define CENTERALIGNED		0x01

/* PWM Scaled */
#define SCALEDCLOCK			0xFF
#define CLOCK				0x00

/*****************************FUNCTION PROTOTYPES*****************************/
void PWMInit(uint8 u8Clock, uint8 u8ScaledValue);
void PWMStart(uint8 u8Channel, uint8 u8Period, uint8 u8Duty,
			  uint8 u8Polarity, uint8 u8Alignment);

/************************************MAIN*************************************/

void main()
{	
	/* configure PWM */
	PWMInit(SCALEDCLOCK,208);
	/* show the output for selected channel */

	/* all PWM channels are @ 30 Hz */
	
	/* even channels through LEDs in EVB */			/* duty cycle */
	PWMStart(0,200,100,HIGHPOL,LEFTALIGNED);		/* 50 %   */			
	PWMStart(2,200,50,HIGHPOL,LEFTALIGNED);			/* 25 %   */
	PWMStart(4,200,25,HIGHPOL,LEFTALIGNED);			/* 12.5 % */
	PWMStart(6,200,12,HIGHPOL,LEFTALIGNED);			/* 6 %    */
	
	/* odd channels through external pins in EVB */
	PWMStart(1,200,200,HIGHPOL,LEFTALIGNED);		/* 100 %  */
	PWMStart(3,200,150,HIGHPOL,LEFTALIGNED);		/* 75 %   */
	PWMStart(5,200,100,HIGHPOL,LEFTALIGNED);		/* 50 %   */
	PWMStart(7,200,1,HIGHPOL,LEFTALIGNED);			/* 0.5 %  */

	for(;;)
	; /* infinite loop */
}
	
/**********************************FUNCTIONS**********************************/

/*****************************************************************************
 * PWMInit: Set channels ready to show a PWM output in pins				     *
 *                                                                           *
 * Parameters: u8Clock:       clock or sclaled clock source					 *
 *			   u8ScaledValue: scaled value to use							 *
 *                                                                           *
 * Return : none.                                                            *
 *****************************************************************************/
void PWMInit(uint8 u8Clock, uint8 u8ScaledValue) 
{
	/* associate odd PWM channels to a pin */
	MCF_GPIO_PTDPAR = 0
					| MCF_GPIO_PTDPAR_PTDPAR0	/* PWM channel 1 */
					| MCF_GPIO_PTDPAR_PTDPAR1	/* PWM channel 3 */
					| MCF_GPIO_PTDPAR_PTDPAR2	/* PWM channel 5 */
					| MCF_GPIO_PTDPAR_PTDPAR3;	/* PWM channel 7 */
	
	/* associate even PWM channels to a pin */
	MCF_GPIO_PTCPAR = 0
					| MCF_GPIO_PTCPAR_TIN0_PWM0	/* PWM channel 0 */
					| MCF_GPIO_PTCPAR_TIN1_PWM2 /* PWM channel 2 */
					| MCF_GPIO_PTCPAR_TIN2_PWM4 /* PWM channel 4 */
					| MCF_GPIO_PTCPAR_TIN3_PWM6;/* PWM channel 6 */
	
	/* associate every channel with a source clock */
	/* A, or B, SA or SB depending the channel */
	MCF_PWM_PWMCLK  = u8Clock;	
	/* change the prescaler of the clock source for every channel */	
	MCF_PWM_PWMPRCLK = 0
					 | MCF_PWM_PWMPRCLK_PCKA(BUSCLK_16) 
					 | MCF_PWM_PWMPRCLK_PCKB(BUSCLK_16);
	/* set scaled value for scaled clock, if are activated */
	MCF_PWM_PWMSCLA = u8ScaledValue;
	MCF_PWM_PWMSCLB = u8ScaledValue;
	
	return;	
}

/*****************************************************************************
 * PWMStart: Start selected PWM channel pin with selected duty and polarity  *
 *                                                                           *
 * Parameters: u8Channel:  PWM channel to start								 *
 *			   u8Period:   PWM channel period								 *
 *             u8Duty: 	   PWM channel duty cycle							 *
 *             u8Polarity: PWM channel polarity								 *
 *			   u8Alignment:PWM channel alignment							 *
 *                                                                           *
 * Return : none.                                                            *
 *****************************************************************************/
void PWMStart(uint8 u8Channel, uint8 u8Period, uint8 u8Duty, 
			  uint8 u8Polarity, uint8 u8Alignment)
{	
	/* for every channel */
	MCF_PWM_PWMPER(u8Channel) = u8Period;			/* set PWM period */
	MCF_PWM_PWMDTY(u8Channel) = u8Duty;				/* set PWM duty cycle */
	MCF_PWM_PWMPOL |= (u8Polarity<<u8Channel); 		/* set PWM polarity */
	MCF_PWM_PWMCAE |= (u8Alignment<<u8Channel); 	/* set PWM alignment */
    /* show the PWM channel in its respective pin*/
    MCF_PWM_PWME |= (1<<u8Channel);
	
	return;
}